﻿using SharpChat.Events;
using SharpChat.Packet;
using System;
using System.Linq;

namespace SharpChat.Commands {
    public class WhisperCommand : IChatCommand {
        public bool IsMatch(ChatCommandContext ctx) {
            return ctx.NameEquals("whisper")
                || ctx.NameEquals("msg");
        }

        public void Dispatch(ChatCommandContext ctx) {
            if(ctx.Args.Length < 2) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.COMMAND_FORMAT_ERROR));
                return;
            }

            string whisperUserStr = ctx.Args.FirstOrDefault();
            ChatUser whisperUser = ctx.Chat.Users.FirstOrDefault(u => u.NameEquals(whisperUserStr));

            if(whisperUser == null) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.USER_NOT_FOUND, true, whisperUserStr));
                return;
            }

            if(whisperUser == ctx.User)
                return;

            string whisperStr = string.Join(' ', ctx.Args.Skip(1));
            string whisperChan = ChatUser.GetDMChannelName(ctx.User, whisperUser);
            DateTimeOffset dateTime = DateTimeOffset.Now;

            ctx.Chat.SendTo(whisperUser, new ChatMessageAddPacket(new ChatMessage {
                DateTime = dateTime,
                ChannelName = whisperChan,
                Sender = ctx.User,
                Text = whisperStr,
                Flags = ChatMessageFlags.Private,
            }));
            ctx.Chat.SendTo(ctx.User, new ChatMessageAddPacket(new ChatMessage {
                DateTime = dateTime,
                ChannelName = whisperChan,
                Sender = ctx.User,
                Text = $"{whisperUser.DisplayName} {whisperStr}",
                Flags = ChatMessageFlags.Private,
            }));
        }
    }
}
